# -*- coding: utf-8 -*-
"""

good_code.py with the addition of function tests!
Here it's written with the intention of the functions being
used elsewhere so the main section just runs the tests and 
quits.

This uses the doctest library, part of the Python Standard Library:
    https://docs.python.org/3/library/doctest.html#module-doctest


An alternate approach is the Python unittest library, also in the PSL.

There are many, many others available.

@author: bgregor
"""

# Imports at the top
import math


def median(x):
    ''' Calculate the median of x
    # Put in the calls to the regular Python interpreter
    
    >>> median([4,2,3,1,5,2])
    2.5
    
    >>> median([4,2,3,1,5])
    3
    
    >>> median([1])
    1
    
    >>> median([])
    
    
    '''
    sorted_x = sorted(x)
    # if length 1 just return
    len_x = len(x)
    if len_x == 0:
        return None
    elif len_x == 1:
        return x[0]
    # If the length is odd, pick the middle value
    elif len(x) % 2 != 0:
        idx = int(len(x)/2) 
        return sorted_x[idx]
     # Otherwise...x length is even. Return the average of 
     # the two middlemost values.
    idx_r = int(len(x)/2)
    idx_l = idx_r - 1
    val = (sorted_x[idx_l] + sorted_x[idx_r]) / 2 
    return val


def mean(x):
    ''' Calculate the mean of x
    
    >>> mean([1,3,-5])
    -0.3333333333333333
    
    '''
    sum_x = sum(x)
    val = sum_x / len(x)
    return val

def std_dev_sqrd_sum(x):
    ''' Calculate the sum of the squares of the 
        difference from the mean for the elements of x
        
    >>> std_dev_sqrd_sum([0,1,2,3.1,1.1,0.25])
    6.632083333333333
    '''
    # get the mean
    mean_x = mean(x)
    # subtract from each element, square, and sum
    sqrd_sum = sum([(elem - mean_x)**2 for elem in x])
    return sqrd_sum

def pop_std_dev(x):
    ''' Calculate the population standard deviation of x
        That's the squard sum of diff. from the mean divided by N
        
    >>> pop_std_dev([0,1,2,3.1,1.1,0.25])
    1.0513549458780427
        
    '''
    sqrd_sum = std_dev_sqrd_sum(x)
    # Divide by N
    val = sqrd_sum / len(x)
    # return the square root
    return math.sqrt(val)

def std_dev(x):
    ''' Calculate the standard deviation of x
        That's the squard sum of diff. from the mean divided by N-1
    
    >>> std_dev([0,1,2,3.1,1.1,0.25])
    1.151701639604054
        
    '''
    sqrd_sum = std_dev_sqrd_sum(x)
    # divide by N-1 and return the square root
    return math.sqrt(sqrd_sum / (len(x)-1))


if __name__=='__main__':
    ''' Run if called as a script '''
    import doctest
    doctest.testmod(verbose=True)


















