# -*- coding: utf-8 -*-
"""
@author: bgregor
"""

# We'll need a function to determine if a number is 
# even or odd.
def is_even(x):
    ''' x is even if there is no remainder
        when it is divided by 2'''
    return (x % 2)==0


# Open the text file for reading
# Here's one way to do it:
#   f = open('numbers.txt','r')
#   all_lines = f.readlines()
#   f.close()

# Here's a better way. The "with..." statement will 
# always close the file if there's an error in reading 
# it.  
with open('numbers.txt','r') as f:
    # Read the contents into a list of strings
    all_lines = f.readlines()
    # This will always close the file but do it
    # anyway...it's a good habit.
    f.close()
    

# Create odd and even lists
odds=[]
evens=[]


# Now let's loop through the lines of the file.  
# Each line looks like: 64,16,95,79,23
# For each line we need to:
#    split up the line around the commas into different strings
#    convert each string to an integer.
#    test if it's even or odd.
#    Append the number to the evens or odds lists
for line in all_lines:
    # line is now each line in the file in turn as a single string.
    # Use the built-in string function split() to make it into a 
    # list of strings.
    str_vals = line.split(',')
    # str_vals -->  ['64','16','95','79','23']
    # Loop again and convert each one to an int, test, and store
    # Note this is independent of the number of values in the line!
    for one_str in str_vals:
        int_val = int(one_str) # int() is a built-in Python function.
        if is_even(int_val):
            evens.append(int_val)
        else:
            odds.append(int_val)
            
# And that's it! 
print('Number of even values: %s' % (len(evens),))
print('Number of odd values: %s' % (len(odds),))

# Now do some math....what's the average of each?
avg_evens = sum(evens) / len(evens)
avg_odds = sum(evens) / len(odds)
print('Averages.  evens: %s   odds: %s' % (avg_evens,avg_odds))
